import { createSlice, PayloadAction } from '@reduxjs/toolkit';
import type { RootState } from './store';
import { AllLayoutAlgorithms, CytoscapeLayoutAlgorithms } from '@graphpolaris/shared/lib/graph-layout';
import { SchemaUtils } from '../../schema/schema-utils';
import { SchemaFromBackend, SchemaGraph, SchemaGraphology } from '../../schema';
import { s } from 'vitest/dist/env-afee91f0';

/**************************************************************** */

export type MLTypes = 'centrality' | 'linkPrediction' | 'communityDetection' | 'shortestPath';
export const allMLTypes: MLTypes[] = ['centrality', 'linkPrediction', 'communityDetection', 'shortestPath'];
export enum MLTypesEnum {
  CENTRALITY = 'centrality',
  LINK_PREDICTION = 'linkPrediction',
  COMMUNITY_DETECTION = 'communityDetection',
  SHORTEST_PATH = 'shortestPath',
}

export type LinkPredictionInstance = {
  attributes: { jaccard_coefficient: number };
  from: string;
  to: string;
  id: string;
};

export type CommunityDetectionInstance = string[]; // set of ids

export type MLInstance<T> = {
  enabled: boolean;
  result: T;
};

export type CommunityDetection = MLInstance<CommunityDetectionInstance[]> & {
  jaccard_threshold: number;
};

export type ShortestPath = {
  enabled: boolean;
  result: any;
  srcNode?: string;
  trtNode?: string;
};

export type ML = {
  [MLTypesEnum.LINK_PREDICTION]: MLInstance<LinkPredictionInstance[]>;
  [MLTypesEnum.CENTRALITY]: MLInstance<Record<string, number>>;
  [MLTypesEnum.COMMUNITY_DETECTION]: CommunityDetection;
  [MLTypesEnum.SHORTEST_PATH]: ShortestPath;
};

// Define the initial state using that type
export const mlDefaultState: ML = {
  linkPrediction: { enabled: false, result: [] },
  centrality: { enabled: false, result: {} },
  communityDetection: { enabled: false, result: [], jaccard_threshold: 0.5 },
  shortestPath: { enabled: false, result: [] },
};
export const mlSlice = createSlice({
  name: 'ml',
  // `createSlice` will infer the state type from the `initialState` argument
  initialState: mlDefaultState,
  reducers: {
    setLinkPredictionEnabled: (state, action: PayloadAction<boolean>) => {
      state.linkPrediction.enabled = action.payload;
    },
    setCommunityDetectionEnabled: (state, action: PayloadAction<boolean>) => {
      state.communityDetection.enabled = action.payload;
    },
    setCentralityEnabled: (state, action: PayloadAction<boolean>) => {
      state.centrality.enabled = action.payload;
    },
    setShortestPathEnabled: (state, action: PayloadAction<boolean>) => {
      state.shortestPath.enabled = action.payload;
    },
    setShortestPathSource: (state, action: PayloadAction<string | undefined>) => {
      state.shortestPath.srcNode = action.payload;
    },
    setShortestPathTarget: (state, action: PayloadAction<string | undefined>) => {
      state.shortestPath.trtNode = action.payload;
    },
    setMLResult: (state, action: PayloadAction<{ type: MLTypes; result: any[] }>) => {
      state[action.payload.type].result = action.payload.result;
    },
  },
});
export const {
  setMLResult,
  setLinkPredictionEnabled,
  setCommunityDetectionEnabled,
  setCentralityEnabled,
  setShortestPathSource,
  setShortestPathTarget,
  setShortestPathEnabled,
} = mlSlice.actions;

export const allMLEnabled = (state: RootState): string => {
  return JSON.stringify(Object.values(state.ml).map((ml) => ml.enabled));
};

export const selectML = (state: RootState) => state.ml;
export default mlSlice.reducer;
